/*
    roxterm - GTK+ 2.0 terminal emulator with tabs
    Copyright (C) 2004 Tony Houghton <h@realh.co.uk>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "defns.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "globalopts.h"
#include "version.h"

Options *global_options = NULL;

char **global_options_commandv = NULL;
char *global_options_appdir = NULL;
char *global_options_bindir = NULL;
char *global_options_directory = NULL;
gboolean global_options_replace = FALSE;

static void global_options_show_usage(void)
{
	puts("roxterm [-?|--help] [--usage] [--geometry=GEOMETRY] [--appdir=DIR]\n"
			"    [--profile=PROFILE|-p PROFILE] [--colour_scheme=SCHEME|-c SCHEME]\n"
			"    [--shortcut_scheme=SCHEME|-s SCHEME]\n"
			"    [--separate] [--replace] [--directory=DIRECTORY|-d DIRECTORY]\n"
			"    [--show-menubar] [--hide-menubar] [-e|--execute COMMAND]\n");
}

static void global_options_show_help(void)
{
	puts("This is " PACKAGE_STRING "\n"
		"Usage: roxterm [OPTION...]\n"
		"Help options\n"
		"  -?, --help                           Show this help message.\n"
		"  --usage                              Show brief usage message.\n\n"
		"Application options\n"
		"  -e, --execute                        Execute remainder of command line\n"
		"                                       inside the terminal. Must be final\n"
		"                                       option.\n"
		"  -d DIRECTORY, --directory=DIRECTORY  Run shell/command in DIRECTORY.\n"
		"  --geometry=GEOMETRY                  Set size and/or position of terminal\n"
		"                                       according to X geometry specification.\n"
		"  --appdir=DIR                         Application directory for ROX.\n"
		"  --show-menubar                       Whether to show or hide the menubar,\n"
		"  --hide-menubar                       overriding the profile. The default\n"
		"                                       is to show it.\n"
		"  --profile=PROFILE, -p PROFILE        Use the named profile.\n"
		"  --colour_scheme=SCHEME, -c SCHEME    Use the named colour scheme.\n"
		"  --shortcut_scheme=SCHEME, -s SCHEME  Use the named keyboard shortcut\n"
		"                                       (accelerator) scheme.\n"
		"  --separate                           Use a separate process for this\n"
		"                                       terminal.\n"
		"  --replace                            Replace any existing instance as\n"
		"                                       ROXTerm D-BUS service provider.\n"
		);
}

/* FIXME: Lots of scrappy bits of code here which could probably be
 * rationalised into fewer, more versatile functions, getopt_long style */

gboolean global_options_preparse_argv_for_execute(int *argc, char **argv,
		gboolean shallow_copy)
{
	int n;

	if (global_options_commandv)
	{
		g_strfreev(global_options_commandv);
		global_options_commandv = NULL;
	}
	for (n = 0; n < *argc; ++n)
	{
		if (!strcmp(argv[n], "--execute") || !strcmp(argv[n], "-e"))
		{
			int m;
			int old_argc = *argc;

			if (n == *argc - 1)
			{
				/* g_error's use of abort is a bit extreme,
				 * use g_critical and exit instead */
				g_critical(_("Missing command after -e/--execute option"));
				return FALSE;
			}

			if (shallow_copy)
				g_free(argv[n]);
			
			global_options_commandv = g_new(char *, *argc - n);

			*argc = n;
			argv[n++] = NULL;
			for (m = 0; n < old_argc; ++m, ++n)
			{
				global_options_commandv[m]
					= shallow_copy ? argv[n] : g_strdup(argv[n]);
			}
			global_options_commandv[m] = NULL;
			break;
		}
	}
	return TRUE;
}

/* Generally we refuse an option parameter beginning with - in case it's a new
 * option and the parameter is missing, but sometimes you may want to begin a
 * parameter with a dash */
static char *global_options_get_param(int *n, int argc, char **argv,
	gboolean accept_dash)
{
	if (argv[*n][1] == '-')
	{
		char *param = strchr(argv[*n], '=');

		if (param && param[1])
		{
			return param + 1;
		}
		else
		{
			g_critical(_("Missing argument to option %s"), argv[*n]);
			return NULL;
		}
	}
	if (++*n >= argc || (!accept_dash && argv[*n][0] == '-'))
	{
		g_critical(_("Missing argument to option %s"), argv[--*n]);
		return NULL;
	}
	return argv[*n];
}

/* Extracts a key/value pair from a command-line option; if param_out is
 * non-NULL it's used to return a copy of the parameter value, otherwise the
 * key/value pair are added to the global hash. Returns FALSE if option is
 * valid but doesn't match ie TRUE means don't try to find any more matches for
 * this option */
static gboolean
global_options_parse_key_value(int *n, int argc, char **argv,
	const char *key, char short_opt, gboolean accept_dash, char **param_out)
{
	char *opt = argv[*n];

	if (opt[0] != '-')
	{
		g_critical(_("Bad parameter %s"), opt);
		return TRUE;
	}
	if ((opt[1] == short_opt && opt[2] == 0) ||
		(opt[1] == '-' && g_str_has_prefix(opt + 2, key) &&
			opt[strlen(key) + 2] == '='))
	{
		char *param = global_options_get_param(n, argc, argv, accept_dash);

		if (param_out)
			*param_out = g_strdup(param);
		else if (param)
			options_set_string(global_options, key, param);
		return TRUE;
	}
	return FALSE;
}

static void global_options_parse_argv(int argc, char **argv)
{
	int n;

	g_free(global_options_directory);
	global_options_directory = NULL;
	for (n = 1; n < argc; ++n)
	{
		if (!strcmp(argv[n], "--help") || !strcmp(argv[n], "-?") ||
			!strcmp(argv[n], "-h"))
		{
			global_options_show_help();
			exit(0);
		}
		else if (!strcmp(argv[n], "--usage"))
		{
			global_options_show_usage();
			exit(0);
		}
		else if (g_str_has_prefix(argv[n], "--geometry="))
		{
			const char *param = global_options_get_param(&n, argc, argv,
				FALSE);

			if (param)
			{
				options_set_string(global_options, "geometry", param);
			}
		}
		else if (!strcmp(argv[n], "--hide-menubar"))
		{
			options_set_int(global_options, "hide_menubar", 1);
		}
		else if (!strcmp(argv[n], "--show-menubar"))
		{
			options_set_int(global_options, "hide_menubar", 0);
		}
		else if (!strcmp(argv[n], "--separate"))
		{
			options_set_int(global_options, "separate", 1);
		}
		else if (!strcmp(argv[n], "--replace"))
		{
			options_set_int(global_options, "replace", 1);
		}
		else if (global_options_parse_key_value(&n, argc, argv,
			"profile", 'p', FALSE, NULL))
		{
			continue;
		}
		else if (global_options_parse_key_value(&n, argc, argv,
			"colour_scheme", 'c', FALSE, NULL))
		{
			continue;
		}
		else if (global_options_parse_key_value(&n, argc, argv,
			"shortcut_scheme", 's', FALSE, NULL))
		{
			continue;
		}
		else if (global_options_parse_key_value(&n, argc, argv,
			"directory", 'd', TRUE, &global_options_directory))
		{
			if (!g_path_is_absolute(global_options_directory))
			{
				char *old = global_options_directory;
				char *cwd = g_get_current_dir();

				global_options_directory = g_build_filename(cwd, old, NULL);
				g_free(cwd);
				g_free(old);
			}
			if (!g_file_test(global_options_directory, G_FILE_TEST_IS_DIR))
			{
				g_warning(_("Invalid directory '%s'"),
						global_options_directory);
				g_free(global_options_directory);
				global_options_directory = NULL;
			}
			continue;
		}
	}
}

void global_options_init_appdir(int argc, char **argv)
{
	int n;

	for (n = 1; n < argc; ++n)
	{
		if (argv[n][0] == '-'
				&& !global_options_parse_key_value(&n, argc, argv, "appdir",
					0, TRUE, &global_options_appdir) && global_options_appdir)
		{
				break;
		}
	}
}

void global_options_init(int argc, char **argv)
{
	static gboolean already = FALSE;

	if (!already)
	{
		global_options_init_appdir(argc, argv);
		/* Need to get appdir before trying to load any options files */
		global_options = options_open("Global", "roxterm options", FALSE);
	}
	global_options_parse_argv(argc, argv);
	if (!already)
	{
		global_options_bindir = g_path_get_dirname(argv[0]);
		if (!strcmp(global_options_bindir, "."))
		{
			/* No directory component, try to find roxterm-config in BIN_DIR,
			 * current dir, then PATH */
			char *capplet;

			g_free(global_options_bindir);
			global_options_bindir = NULL;
			capplet = g_build_filename(BIN_DIR, "roxterm-config", NULL);
			if (g_file_test(capplet, G_FILE_TEST_IS_EXECUTABLE))
			{
				global_options_bindir = g_strdup(BIN_DIR);
			}
			else if (g_file_test("roxterm-config", G_FILE_TEST_IS_EXECUTABLE))
			{
				global_options_bindir = g_get_current_dir();
			}
			else if (!g_find_program_in_path("roxterm-config"))
			{
				g_critical(_("Can't find roxterm-config"));
			}
		}
		else if (!g_path_is_absolute(global_options_bindir))
		{
			/* Partical path, must be relative to current dir */
			char *cur = g_get_current_dir();
			char *full = g_build_filename(cur, global_options_bindir, NULL);

			g_free(cur);
			g_free(global_options_bindir);
			global_options_bindir = full;
		}
		/* else full path was given */
	}
	already = TRUE;
}

char **global_options_copy_strv(char **ps)
{
	char **ps2;
	int n;

	for (n = 0; ps[n]; ++n);
	ps2 = g_new(char *, n + 1);
	for (; n >= 0; --n)
		ps2[n] = ps[n] ? g_strdup(ps[n]) : NULL;
	return ps2;
}

/* vi:set sw=4 ts=4 noet cindent cino= */
